"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.XrpFullBlock = exports.UtxoFullBlock = exports.LtcFullBlock = exports.DogeFullBlock = exports.BtcFullBlock = exports.XrpBlock = exports.UtxoBlock = exports.LtcBlock = exports.DogeBlock = exports.BtcBlock = exports.AlgoBlock = exports.UtxoBlockTip = exports.LtcBlockTip = exports.DogeBlockTip = exports.BtcBlockTip = exports.UtxoBlockHeader = exports.LtcBlockHeader = exports.DogeBlockHeader = exports.BtcBlockHeader = exports.fullBlockConstructor = exports.blockTipConstructor = exports.blockHeaderConstructor = exports.blockConstructor = exports.FullBlockBase = exports.BlockBase = exports.BlockHeaderBase = exports.BlockTipBase = void 0;
class BlockTipBase {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    constructor(data) {
        this.privateData = data;
    }
    /**
     * Exposing the private data for the derived classes (dev only/python like privatization)
     */
    get _data() {
        return this.privateData;
    }
    /**
     * Only return something on Tips
     */
    get chainTipStatus() {
        return "active";
    }
}
exports.BlockTipBase = BlockTipBase;
class BlockHeaderBase extends BlockTipBase {
}
exports.BlockHeaderBase = BlockHeaderBase;
class BlockBase extends BlockHeaderBase {
    /**
     * Return if block is valid
     * Mainly for XRP
     */
    get isValid() {
        return true;
    }
}
exports.BlockBase = BlockBase;
/**
 * Base class for blocks that also include all transactions (including information about the transaction)
 */
class FullBlockBase extends BlockBase {
}
exports.FullBlockBase = FullBlockBase;
function blockConstructor(c) {
    return new c();
}
exports.blockConstructor = blockConstructor;
function blockHeaderConstructor(c) {
    return new c();
}
exports.blockHeaderConstructor = blockHeaderConstructor;
function blockTipConstructor(c) {
    return new c();
}
exports.blockTipConstructor = blockTipConstructor;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function fullBlockConstructor(c) {
    return new c();
}
exports.fullBlockConstructor = fullBlockConstructor;
// Block Tips
/**
 * Algo and Ripple (XRP) have no specific block header endpoint (liter block with limited data)
 */
// Block headers
var BtcBlockHeader_1 = require("./blockHeaders/BtcBlockHeader");
Object.defineProperty(exports, "BtcBlockHeader", { enumerable: true, get: function () { return BtcBlockHeader_1.BtcBlockHeader; } });
var DogeBlockHeader_1 = require("./blockHeaders/DogeBlockHeader");
Object.defineProperty(exports, "DogeBlockHeader", { enumerable: true, get: function () { return DogeBlockHeader_1.DogeBlockHeader; } });
var LtcBlockHeader_1 = require("./blockHeaders/LtcBlockHeader");
Object.defineProperty(exports, "LtcBlockHeader", { enumerable: true, get: function () { return LtcBlockHeader_1.LtcBlockHeader; } });
var UtxoBlockHeader_1 = require("./blockHeaders/UtxoBlockHeader");
Object.defineProperty(exports, "UtxoBlockHeader", { enumerable: true, get: function () { return UtxoBlockHeader_1.UtxoBlockHeader; } });
var BtcBlockTip_1 = require("./blockTips/BtcBlockTip");
Object.defineProperty(exports, "BtcBlockTip", { enumerable: true, get: function () { return BtcBlockTip_1.BtcBlockTip; } });
var DogeBlockTip_1 = require("./blockTips/DogeBlockTip");
Object.defineProperty(exports, "DogeBlockTip", { enumerable: true, get: function () { return DogeBlockTip_1.DogeBlockTip; } });
var LtcBlockTip_1 = require("./blockTips/LtcBlockTip");
Object.defineProperty(exports, "LtcBlockTip", { enumerable: true, get: function () { return LtcBlockTip_1.LtcBlockTip; } });
var UtxoBlockTip_1 = require("./blockTips/UtxoBlockTip");
Object.defineProperty(exports, "UtxoBlockTip", { enumerable: true, get: function () { return UtxoBlockTip_1.UtxoBlockTip; } });
// Blocks
var AlgoBlock_1 = require("./blocks/AlgoBlock");
Object.defineProperty(exports, "AlgoBlock", { enumerable: true, get: function () { return AlgoBlock_1.AlgoBlock; } });
var BtcBlock_1 = require("./blocks/BtcBlock");
Object.defineProperty(exports, "BtcBlock", { enumerable: true, get: function () { return BtcBlock_1.BtcBlock; } });
var DogeBlock_1 = require("./blocks/DogeBlock");
Object.defineProperty(exports, "DogeBlock", { enumerable: true, get: function () { return DogeBlock_1.DogeBlock; } });
var LtcBlock_1 = require("./blocks/LtcBlock");
Object.defineProperty(exports, "LtcBlock", { enumerable: true, get: function () { return LtcBlock_1.LtcBlock; } });
var UtxoBlock_1 = require("./blocks/UtxoBlock");
Object.defineProperty(exports, "UtxoBlock", { enumerable: true, get: function () { return UtxoBlock_1.UtxoBlock; } });
var XrpBlock_1 = require("./blocks/XrpBlock");
Object.defineProperty(exports, "XrpBlock", { enumerable: true, get: function () { return XrpBlock_1.XrpBlock; } });
// Full blocks
var BtcFullBlock_1 = require("./fullBlocks/BtcFullBlock");
Object.defineProperty(exports, "BtcFullBlock", { enumerable: true, get: function () { return BtcFullBlock_1.BtcFullBlock; } });
var DogeFullBlock_1 = require("./fullBlocks/DogeFullBlock");
Object.defineProperty(exports, "DogeFullBlock", { enumerable: true, get: function () { return DogeFullBlock_1.DogeFullBlock; } });
var LtcFullBlock_1 = require("./fullBlocks/LtcFullBlock");
Object.defineProperty(exports, "LtcFullBlock", { enumerable: true, get: function () { return LtcFullBlock_1.LtcFullBlock; } });
var UtxoFullBlock_1 = require("./fullBlocks/UtxoFullBlock");
Object.defineProperty(exports, "UtxoFullBlock", { enumerable: true, get: function () { return UtxoFullBlock_1.UtxoFullBlock; } });
var XrpFullBlock_1 = require("./fullBlocks/XrpFullBlock");
Object.defineProperty(exports, "XrpFullBlock", { enumerable: true, get: function () { return XrpFullBlock_1.XrpFullBlock; } });
//# sourceMappingURL=BlockBase.js.map