"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlgoIndexerTransaction = void 0;
const types_1 = require("../../types");
const algoUtils_1 = require("../../utils/algoUtils");
const constants_1 = require("../../utils/constants");
const errors_1 = require("../../utils/errors");
const managed_1 = require("../../utils/managed");
const utils_1 = require("../../utils/utils");
const TransactionBase_1 = require("../TransactionBase");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const web3 = require("web3");
let AlgoIndexerTransaction = class AlgoIndexerTransaction extends TransactionBase_1.TransactionBase {
    get data() {
        return this.privateData;
    }
    get additionalData() {
        return this.privateAdditionalData;
    }
    set additionalData(data) {
        this.privateAdditionalData = data;
    }
    get intendedSpentAmounts() {
        throw new Error("Method not implemented.");
    }
    get intendedReceivedAmounts() {
        throw new Error("Method not implemented.");
    }
    async makeFull(client) {
        if (!this.additionalData) {
            this.additionalData = {};
        }
        if (!this.additionalData.assetInfo) {
            if (this.data.transaction.assetTransferTransaction) {
                this.additionalData.assetInfo = await client.getIndexerAssetInfo(this.data.transaction.assetTransferTransaction.assetId);
            }
        }
    }
    get txid() {
        return this.hash;
    }
    get stdTxid() {
        return (0, algoUtils_1.txIdToHexNo0x)(this.txid);
    }
    get hash() {
        if (this.data.transaction.id === undefined) {
            return "";
        }
        return this.data.transaction.id;
    }
    get reference() {
        if (this.data.transaction.note) {
            return [(0, algoUtils_1.base64ToHex)(this.data.transaction.note)];
        }
        return [];
    }
    get stdPaymentReference() {
        let paymentReference = this.reference.length === 1 ? (0, utils_1.prefix0x)(this.reference[0]) : "";
        try {
            // try to parse out
            paymentReference = (0, utils_1.prefix0x)(web3.utils.hexToString((0, utils_1.prefix0x)(paymentReference)));
        }
        catch (e) {
            return utils_1.ZERO_BYTES_32;
        }
        if (!(0, utils_1.isValidBytes32Hex)(paymentReference)) {
            paymentReference = utils_1.ZERO_BYTES_32;
        }
        return paymentReference;
    }
    get unixTimestamp() {
        if (this.data.transaction.roundTime === undefined) {
            return 0;
        }
        return this.data.transaction.roundTime;
    }
    get sourceAddresses() {
        return [this.data.transaction.sender];
    }
    get assetSourceAddresses() {
        if (this.type === "axfer" || this.type === "axfer_close") {
            // for token transfers send by clawback transaction (https://developer.algorand.org/docs/get-details/transactions/transactions/#asset-clawback-transaction)
            // and asset transfer transactions (https://developer.algorand.org/docs/get-details/transactions/transactions/#asset-transfer-transaction)
            if (this.data.transaction.assetTransferTransaction.sender) {
                // for asset clawback tx it somehow gets addess w/out checksum
                return [this.data.transaction.assetTransferTransaction.sender];
            }
            // Other kinds of asset transactions (axfer and axfer_close ) use snd field as sender
            return [this.data.transaction.sender];
        }
        // Other kinds of transactions have nothing to do with assets
        return [];
    }
    // TODO close address
    get receivingAddresses() {
        if (this.type === "pay" && this.data.transaction.paymentTransaction) {
            return [this.data.transaction.paymentTransaction.receiver];
        }
        // TODO check other transaction types
        return [];
    }
    // TODO what if revicer=closeTo
    get assetReceivingAddresses() {
        const recAddresses = [];
        if (this.type === "axfer" || this.type === "axfer_close") {
            if (this.data.transaction.assetTransferTransaction.receiver) {
                recAddresses.push(this.data.transaction.assetTransferTransaction.receiver);
            }
            if (this.data.transaction.assetTransferTransaction.closeTo) {
                recAddresses.push(this.data.transaction.assetTransferTransaction.closeTo);
            }
            return recAddresses;
        }
        return [];
    }
    get fee() {
        return (0, utils_1.toBN)(this.data.transaction.fee);
    }
    get feeSignerTotalAmount() {
        throw new Error("Method not implemented.");
    }
    get spentAmounts() {
        if (this.data.transaction.txType === "pay" && this.data.transaction.paymentTransaction) {
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee.add((0, utils_1.toBN)(this.data.transaction.paymentTransaction.amount)),
                },
            ];
        }
        else {
            return [
                {
                    address: this.sourceAddresses[0],
                    amount: this.fee,
                },
            ];
        }
    }
    get assetSpentAmounts() {
        // for transactions of type axfer
        if (this.type === "axfer") {
            if (this.data.transaction.assetTransferTransaction) {
                let preamount = this.data.transaction.assetTransferTransaction.amount;
                if (this.data.transaction.assetTransferTransaction.closeAmount) {
                    preamount += this.data.transaction.assetTransferTransaction.closeAmount;
                }
                const amount = preamount.toString();
                if (!(this.additionalData && this.additionalData.assetInfo)) {
                    throw new errors_1.mccError(errors_1.mccErrorCode.InvalidResponse, Error("call the makeFull before to make sure additional data full"));
                }
                let decimals = this.additionalData.assetInfo.params.decimals;
                if (typeof decimals === "bigint") {
                    decimals = Number(decimals);
                }
                return [
                    {
                        address: this.assetSourceAddresses[0],
                        amount: (0, utils_1.toBN)(amount),
                        elementaryUnits: web3.utils.toBN(Math.pow(10, decimals)),
                    },
                ];
            }
        }
        return [];
        // For now we don't support spending / using non-native tokens
    }
    // TODO close amount
    // TODO what if receiver=closeTo
    get receivedAmounts() {
        if (this.type === "pay" && this.data.transaction.paymentTransaction) {
            return [
                {
                    address: this.receivingAddresses[0],
                    amount: (0, utils_1.toBN)(this.data.transaction.paymentTransaction.amount),
                },
            ];
        }
        return [];
    }
    // TODO what if receiver=closeTo
    get assetReceivedAmounts() {
        const asRecAm = [];
        if (this.type === "axfer") {
            if (this.data.transaction.assetTransferTransaction) {
                const amount = this.data.transaction.assetTransferTransaction.amount.toString();
                if (!(this.additionalData && this.additionalData.assetInfo)) {
                    throw new errors_1.mccError(errors_1.mccErrorCode.InvalidResponse, Error("call the makeFull before to make sure additional data is full"));
                }
                let decimals = this.additionalData.assetInfo.params.decimals;
                if (typeof decimals === "bigint") {
                    decimals = Number(decimals);
                }
                asRecAm.push({
                    address: this.data.transaction.assetTransferTransaction.receiver,
                    amount: (0, utils_1.toBN)(amount),
                    elementaryUnits: web3.utils.toBN(Math.pow(10, decimals)),
                });
                if (this.data.transaction.assetTransferTransaction.closeTo) {
                    const clAmount = this.data.transaction.assetTransferTransaction.closeAmount.toString();
                    asRecAm.push({
                        address: this.data.transaction.assetTransferTransaction.closeTo,
                        amount: (0, utils_1.toBN)(clAmount),
                        elementaryUnits: web3.utils.toBN(Math.pow(10, decimals)),
                    });
                }
                return asRecAm;
            }
        }
        return [];
    }
    get type() {
        return this.data.transaction.txType;
    }
    get isNativePayment() {
        return this.type === "pay";
    }
    get currencyName() {
        if (this.type === "pay") {
            return constants_1.ALGO_NATIVE_TOKEN_NAME;
        }
        else if (this.type === "axfer" && this.data.transaction.assetTransferTransaction) {
            return this.data.transaction.assetTransferTransaction.assetId.toString();
        }
        return "";
    }
    get elementaryUnits() {
        return (0, utils_1.toBN)(constants_1.ALGO_MDU);
    }
    get successStatus() {
        // TODO research this further
        return types_1.TransactionSuccessStatus.SUCCESS;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    async paymentSummary(props) {
        throw new Error("Method not implemented.");
        // if (!this.isNativePayment) {
        //    return { isNativePayment: false };
        // }
        // return {
        //    isNativePayment: true,
        //    sourceAddress: this.sourceAddresses[0],
        //    receivingAddress: this.receivingAddresses[0],
        //    spentAmount: this.spentAmounts[0].amount,
        //    receivedAmount: this.receivedAmounts[0].amount,
        //    paymentReference: this.stdPaymentReference,
        //    oneToOne: true,
        //    isFull: true,
        // };
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    balanceDecreasingSummary(props) {
        throw new Error("Method not implemented.");
    }
};
AlgoIndexerTransaction = __decorate([
    (0, managed_1.Managed)()
], AlgoIndexerTransaction);
exports.AlgoIndexerTransaction = AlgoIndexerTransaction;
//# sourceMappingURL=AlgoIndexerTransaction.js.map