import BN from "bn.js";
import { MccClient, TransactionSuccessStatus } from "../../types";
import { AlgoTransactionTypeOptions, IAlgoAdditionalData, IAlgoTransactionMsgPack } from "../../types/algoTypes";
import { AddressAmount, BalanceDecreasingProps, BalanceDecreasingSummaryResponse, PaymentSummaryProps, PaymentSummaryResponse, TransactionBase } from "../TransactionBase";
/**
 * docs https://developer.algorand.org/docs/get-details/transactions/transactions/
 */
export declare class AlgoTransaction extends TransactionBase {
    protected get data(): IAlgoTransactionMsgPack;
    protected get additionalData(): IAlgoAdditionalData | undefined;
    protected set additionalData(data: IAlgoAdditionalData | undefined);
    get intendedSpentAmounts(): AddressAmount[];
    get intendedReceivedAmounts(): AddressAmount[];
    get txid(): string;
    get stdTxid(): string;
    get hash(): string;
    get reference(): string[];
    get stdPaymentReference(): string;
    get unixTimestamp(): number;
    get sourceAddresses(): (string | undefined)[];
    get assetSourceAddresses(): (string | undefined)[];
    get receivingAddresses(): (string | undefined)[];
    get assetReceivingAddresses(): (string | undefined)[];
    get fee(): BN;
    get feeSignerTotalAmount(): AddressAmount;
    get spentAmounts(): AddressAmount[];
    get assetSpentAmounts(): AddressAmount[];
    get receivedAmounts(): AddressAmount[];
    get assetReceivedAmounts(): AddressAmount[];
    get type(): AlgoTransactionTypeOptions;
    get isNativePayment(): boolean;
    get currencyName(): string;
    get elementaryUnits(): BN;
    get successStatus(): TransactionSuccessStatus;
    paymentSummary(props: PaymentSummaryProps): Promise<PaymentSummaryResponse>;
    makeFull(client: MccClient): Promise<void>;
    balanceDecreasingSummary(props: BalanceDecreasingProps): Promise<BalanceDecreasingSummaryResponse>;
    private getTransactionAssetParams;
}
//# sourceMappingURL=AlgoTransaction.d.ts.map