import { AccountInfoRequest, AccountTxRequest, LedgerRequest, LedgerResponse, TxResponse } from "xrpl";
import { RateLimitOptions } from "../types";
import { IIGetBlockRes, IIGetTransactionRes, MccLoggingOptions } from "./genericMccTypes";
export declare class XrpMccCreate {
    url: string;
    apiTokenKey?: string;
    username?: string;
    password?: string;
    inRegTest?: boolean;
    rateLimitOptions?: RateLimitOptions | undefined;
    loggingOptions?: MccLoggingOptions | undefined;
}
export interface XrpCreateAddressData {
}
export interface IXrpGetTransactionRes extends IIGetTransactionRes, TxResponse {
}
export interface IXrpGetFullTransactionRes extends IXrpGetTransactionRes {
}
export interface IXrpGetBlockRes extends LedgerResponse, IIGetBlockRes {
}
export interface IXrpGetBlockHeaderRes extends LedgerResponse, IIGetBlockRes {
}
export declare type XrpTransactionStatusPrefixes = "tes" | "ter" | "tem" | "tel" | "tef" | "tec";
export declare type XrpTransactionStatusTec = "tecCANT_ACCEPT_OWN_NFTOKEN_OFFER" | "tecCLAIM" | "tecCRYPTOCONDITION_ERROR" | "tecDIR_FULL" | "tecDUPLICATE" | "tecDST_TAG_NEEDED" | "tecEXPIRED" | "tecFAILED_PROCESSING" | "tecFROZEN" | "tecHAS_OBLIGATIONS" | "tecINSUF_RESERVE_LINE" | "tecINSUF_RESERVE_OFFER" | "tecINSUFF_FEE" | "tecINSUFFICIENT_FUNDS" | "tecINSUFFICIENT_PAYMENT" | "tecINSUFFICIENT_RESERVE" | "tecINTERNAL" | "tecINVARIANT_FAILED" | "tecKILLED" | "tecMAX_SEQUENCE_REACHED" | "tecNEED_MASTER_KEY" | "tecNFTOKEN_BUY_SELL_MISMATCH" | "tecNFTOKEN_OFFER_TYPE_MISMATCH" | "tecNO_ALTERNATIVE_KEY" | "tecNO_AUTH" | "tecNO_DST" | "tecNO_DST_INSUF_XRP" | "tecNO_ENTRY" | "tecNO_ISSUER" | "tecNO_LINE" | "tecNO_LINE_INSUF_RESERVE" | "tecNO_LINE_REDUNDANT" | "tecNO_PERMISSION" | "tecNO_REGULAR_KEY" | "tecNO_SUITABLE_NFTOKEN_PAGE" | "tecNO_TARGET" | "tecOBJECT_NOT_FOUND" | "tecOVERSIZE" | "tecOWNERS" | "tecPATH_DRY" | "tecPATH_PARTIAL" | "tecTOO_SOON" | "tecUNFUNDED" | "tecUNFUNDED_ADD" | "tecUNFUNDED_PAYMENT" | "tecUNFUNDED_OFFER";
export declare type XrpTransactionTypeUnion = "NFTokenAcceptOffer" | "NFTokenBurn" | "NFTokenCancelOffer" | "NFTokenCreateOffer" | "NFTokenMint" | "AccountDelete" | "AccountSet" | "CheckCancel" | "CheckCash" | "CheckCreate" | "DepositPreauth" | "EscrowCancel" | "EscrowCreate" | "EscrowFinish" | "OfferCancel" | "OfferCreate" | "Payment" | "PaymentChannelClaim" | "PaymentChannelCreate" | "PaymentChannelFund" | "SetRegularKey" | "SignerListSet" | "TicketCreate" | "TrustSet";
export declare type AccountRootFlags = "lsfDefaultRipple" | "lsfDepositAuth" | "lsfDisableMaster" | "lsfDisallowXRP" | "lsfGlobalFreeze" | "lsfNoFreeze" | "lsfPasswordSpent" | "lsfRequireAuth" | "lsfRequireDestTag";
export declare const FlagToHex: {
    lsfDepositAuth: number;
    lsfDefaultRipple: number;
    lsfGlobalFreeze: number;
    lsfNoFreeze: number;
    lsfDisableMaster: number;
    lsfDisallowXRP: number;
    lsfRequireAuth: number;
    lsfRequireDestTag: number;
    lsfPasswordSpent: number;
};
export declare const allHexFlags: number[];
export declare const HexToFlag: {
    16777216: string;
    8388608: string;
    4194304: string;
    2097152: string;
    1048576: string;
    524288: string;
    262144: string;
    131072: string;
    65536: string;
};
export declare const PosToFlag: {
    24: string;
    23: string;
    22: string;
    21: string;
    20: string;
    19: string;
    18: string;
    17: string;
    16: string;
};
export declare const SpecialAddressesReverse: {
    rrrrrrrrrrrrrrrrrrrrrhoLvTp: string;
    rrrrrrrrrrrrrrrrrrrrBZbvji: string;
    rHb9CJAWyB4rj91VRWn96DkukG4bwdtyTh: string;
    rrrrrrrrrrrrrrrrrNAMEtxvNvQ: string;
    rrrrrrrrrrrrrrrrrrrn5RM1rHd: string;
};
export declare const SpecialAddresses: {
    ACCOUNT_ZERO: string;
    ACCOUNT_ONE: string;
    GENESIS_ACCOUNT: string;
    RESERVATION_BLACK_HOLE: string;
    NaN_ADDRESS: string;
};
export declare type XrpBlockReqParams = Omit<LedgerRequest, "command">;
export declare type IAccountInfoRequest = Omit<AccountInfoRequest, "command">;
export declare type IAccountTxRequest = Omit<AccountTxRequest, "command">;
//# sourceMappingURL=xrpTypes.d.ts.map