"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.utxo_ensure_data = exports.utxo_check_expect_block_out_of_range = exports.utxo_check_expect_empty = exports.WordToOpcode = void 0;
const utils_1 = require("./utils");
/**
 * Opcode words to their opcodes
 * @dev https://en.bitcoin.it/wiki/Script
 */
var WordToOpcode;
(function (WordToOpcode) {
    WordToOpcode[WordToOpcode["OP_RETURN"] = 106] = "OP_RETURN";
})(WordToOpcode = exports.WordToOpcode || (exports.WordToOpcode = {}));
////////////////////////////
//// MCC Error handling ////
////////////////////////////
/**
 * Check if return error code is -5 (can't find transaction)
 * @param data
 * @returns
 */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_check_expect_empty(data) {
    if (!data || !data.error || !data.error.code) {
        return false;
    }
    if (data.error.code && data.error.code === -5) {
        return true;
    }
    else {
        return false;
    }
}
exports.utxo_check_expect_empty = utxo_check_expect_empty;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_check_expect_block_out_of_range(data) {
    if (data && data.error && data.error.code) {
        if (data.error.code === -8) {
            return true;
        }
        if (data.error.code === -1) {
            return true;
        }
        else {
            return false;
        }
    }
    else {
        return false;
    }
}
exports.utxo_check_expect_block_out_of_range = utxo_check_expect_block_out_of_range;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function utxo_ensure_data(data) {
    if (data === null || data === void 0 ? void 0 : data.error) {
        throw (0, utils_1.MccError)(data);
    }
}
exports.utxo_ensure_data = utxo_ensure_data;
//# sourceMappingURL=utxoUtils.js.map