"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("../../src");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const chai = require("chai");
// eslint-disable-next-line @typescript-eslint/no-var-requires
chai.use(require("chai-as-promised"));
const LtcMccConnection = {
    url: process.env.LTC_URL || "",
    username: process.env.LTC_USERNAME || "",
    password: process.env.LTC_PASSWORD || "",
    apiTokenKey: process.env.FLARE_API_PORTAL_KEY || "",
};
describe("Transaction Ltc base test ", function () {
    let MccClient;
    before(async function () {
        MccClient = new src_1.MCC.LTC(LtcMccConnection);
    });
    describe("Transaction does not exist ", function () {
        const txid = "8bae12b5f4c088d940733dcd1455efc6a3a69cf9340e17a981286d37786156ff";
        it("Should get transaction does not exist ", async function () {
            const transaction = MccClient.getTransaction(txid);
            await (0, chai_1.expect)(transaction).to.be.rejectedWith("InvalidTransaction");
        });
    });
    describe("Transaction not full", function () {
        const txid = "40225226e4a3c7b00eebb8ec8fcb738c7069f557f3dd31370d51388ed65aa908";
        let transaction;
        before(async () => {
            transaction = await MccClient.getTransaction(txid);
        });
        it("Should return undefined for not full transaction for source addresses", async function () {
            (0, chai_1.expect)(transaction.sourceAddresses).to.eq([undefined]);
        });
        it("Should return empty spentAmounts for not full transaction for spentAmounts", async function () {
            (0, chai_1.expect)(transaction.spentAmounts).to.deep.eq({
                amount: (0, src_1.toBN)(0),
            });
        });
        it("Should have correct type", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("Payment");
        });
    });
    describe("Transaction full ", function () {
        const txid = "40225226e4a3c7b00eebb8ec8fcb738c7069f557f3dd31370d51388ed65aa908";
        let transaction;
        before(async () => {
            transaction = await MccClient.getTransaction(txid);
            await transaction.makeFullPayment(MccClient);
        });
        it("Should find transaction in block ", function () {
            (0, chai_1.expect)(transaction).to.not.eq(undefined);
        });
        it("Should get transaction txid ", async function () {
            (0, chai_1.expect)(transaction.txid).to.eq(txid);
        });
        it("Should get standardized txid ", async function () {
            (0, chai_1.expect)(transaction.stdTxid).to.eq(txid);
        });
        it("Should get transaction hash ", async function () {
            (0, chai_1.expect)(transaction.hash).to.eq("5533d6dd7db5b2b0aeccb6f82d576308341ccbaaf78fbb968468cfb25d14ac02");
        });
        it("Should get transaction reference array ", async function () {
            const refarr = [];
            (0, chai_1.expect)(transaction.reference.length).to.eq(refarr.length);
            const a = transaction.reference.sort();
            const b = refarr.sort();
            for (let i = 0; i < a.length; i++) {
                (0, chai_1.expect)(a[i]).to.eq(b[i]);
            }
        });
        it("Should get standardized transaction reference ", async function () {
            (0, chai_1.expect)(transaction.stdPaymentReference).to.eq("0x0000000000000000000000000000000000000000000000000000000000000000");
        });
        it("Should get transaction timestamp ", async function () {
            (0, chai_1.expect)(transaction.unixTimestamp).to.eq(1650010974);
        });
        it("Should get source address ", async function () {
            const sourceAdd = ["MNxic71ZtazMySUJcTt9fShYGy9xgY69PE", "MMiDjEpzzvuuts1G3XrhM6UKcUyPcevcYB"];
            (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(sourceAdd.length);
            const a = transaction.sourceAddresses.sort();
            const b = sourceAdd.sort();
            for (let i = 0; i < a.length; i++) {
                (0, chai_1.expect)(a[i]).to.eq(b[i]);
            }
        });
        it("Should get receiving address ", async function () {
            const recAdd = ["ltc1q4tjs969dneux5cldan034vkfuteelt6l2h4w96", "ltc1q60a8aszz6aqlnac5n6zdmyefyhz7g5lul6h5jj"];
            (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(recAdd.length);
            const a = transaction.receivingAddresses.sort();
            const b = recAdd.sort();
            for (let i = 0; i < a.length; i++) {
                (0, chai_1.expect)(a[i]).to.eq(b[i]);
            }
        });
        it("Should get fee ", async function () {
            (0, chai_1.expect)(transaction.fee.toString()).to.eq("25500");
        });
        it("Should get currency name", async () => {
            (0, chai_1.expect)(transaction.currencyName).to.eq("LTC");
        });
        it("Should spend amount ", async function () {
            const sourceAdd = [
                {
                    address: "MNxic71ZtazMySUJcTt9fShYGy9xgY69PE",
                    amount: (0, src_1.toBN)(6000),
                },
                {
                    address: "MMiDjEpzzvuuts1G3XrhM6UKcUyPcevcYB",
                    amount: (0, src_1.toBN)(14783439),
                },
            ];
            (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(sourceAdd.length);
            const a = transaction.spentAmounts.sort();
            const b = sourceAdd.sort();
            for (let i = 0; i < a.length; i++) {
                (0, chai_1.expect)(a[i].address).to.eq(b[i].address);
                (0, chai_1.expect)(a[i].amount.toString()).to.eq(b[i].amount.toString());
            }
        });
        it("Should received amount ", async function () {
            const recAdd = [
                {
                    address: "ltc1q4tjs969dneux5cldan034vkfuteelt6l2h4w96",
                    amount: (0, src_1.toBN)(13788884),
                },
                {
                    address: "ltc1q60a8aszz6aqlnac5n6zdmyefyhz7g5lul6h5jj",
                    amount: (0, src_1.toBN)(975055),
                },
            ];
            (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(recAdd.length);
            const a = transaction.receivedAmounts.sort();
            const b = recAdd.sort();
            for (let i = 0; i < a.length; i++) {
                (0, chai_1.expect)(a[i].address).to.eq(b[i].address);
                (0, chai_1.expect)(a[i].amount.toString()).to.eq(b[i].amount.toString());
            }
        });
        it("Should get type ", async function () {
            (0, chai_1.expect)(transaction.type).to.eq("full_payment");
        });
    });
    const TransactionsToTest = [
        {
            description: "Transaction with one reference",
            txid: "ac2b5558202927d78aa1f798c325180ccb8ba6b86eec0e3eac0177dbbb0a4eb8",
            expect: {
                txid: "ac2b5558202927d78aa1f798c325180ccb8ba6b86eec0e3eac0177dbbb0a4eb8",
                stdTxid: "ac2b5558202927d78aa1f798c325180ccb8ba6b86eec0e3eac0177dbbb0a4eb8",
                hash: "65b9933f105549cd4196369d30673b41034aa490fb9a915ef7ad3a9c68085339",
                reference: [],
                stdPaymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                unixTimestamp: 1655666878,
                sourceAddresses: [undefined],
                receivingAddresses: ["ltc1qetweexqmrszez23s8l6tlc0qvfgyneyg08q06m", "ML3CdKXQYiBBY8RWcXapuV2ZHpWbMo7DP1"],
                isFeeError: true,
                fee: "InvalidResponse",
                spentAmounts: [
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                receivedAmounts: [
                    {
                        address: "ltc1qetweexqmrszez23s8l6tlc0qvfgyneyg08q06m",
                        amount: (0, src_1.toBN)(545237020),
                    },
                    {
                        address: "ML3CdKXQYiBBY8RWcXapuV2ZHpWbMo7DP1",
                        amount: (0, src_1.toBN)(134761560),
                    },
                ],
                type: "payment",
                isNativePayment: true,
                currencyName: "LTC",
                elementaryUnits: "100000000",
                successStatus: src_1.TransactionSuccessStatus.SUCCESS,
            },
            summary: {
                status: src_1.PaymentSummaryStatus.Success,
            },
        },
        {
            description: "Coinbase Transaction",
            txid: "458584edb759bc2b5842ba52467445c6c8e51dffcac5b5b9c59c8d88fd8447b6",
            expect: {
                txid: "458584edb759bc2b5842ba52467445c6c8e51dffcac5b5b9c59c8d88fd8447b6",
                stdTxid: "458584edb759bc2b5842ba52467445c6c8e51dffcac5b5b9c59c8d88fd8447b6",
                hash: "818e7777f12becb8a61d90b48dbc966aeb45a1bc1221d83db92767f772564043",
                reference: ["aa21a9edc37769589e812ce376b3f0189ca132bf1005b764416ad2d851a6efb2f8021aae"],
                stdPaymentReference: "0x0000000000000000000000000000000000000000000000000000000000000000",
                unixTimestamp: 1655666505,
                sourceAddresses: [undefined],
                receivingAddresses: ["LfmssDyX6iZvbVqHv6t9P6JWXia2JG7mdb", undefined],
                isFeeError: false,
                fee: "0",
                spentAmounts: [
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                receivedAmounts: [
                    {
                        address: "LfmssDyX6iZvbVqHv6t9P6JWXia2JG7mdb",
                        amount: (0, src_1.toBN)(1250889287),
                    },
                    {
                        address: undefined,
                        amount: (0, src_1.toBN)(0),
                    },
                ],
                type: "coinbase",
                isNativePayment: true,
                currencyName: "LTC",
                elementaryUnits: "100000000",
                successStatus: src_1.TransactionSuccessStatus.SUCCESS,
            },
            summary: {
                status: src_1.PaymentSummaryStatus.Success,
            },
        },
    ];
    for (const transData of TransactionsToTest) {
        describe(transData.description, function () {
            let transaction;
            before(async function () {
                transaction = await MccClient.getTransaction(transData.txid);
            });
            it("Should find transaction in block ", function () {
                (0, chai_1.expect)(transaction).to.not.eq(undefined);
            });
            it("Should get transaction txid ", async function () {
                (0, chai_1.expect)(transaction.txid).to.eq(transData.expect.txid);
            });
            it("Should get standardized txid ", async function () {
                (0, chai_1.expect)(transaction.stdTxid).to.eq(transData.expect.stdTxid);
            });
            it("Should get transaction hash ", async function () {
                (0, chai_1.expect)(transaction.hash).to.eq(transData.expect.hash);
            });
            it("Should get transaction reference array ", async function () {
                (0, chai_1.expect)(transaction.reference.length).to.eq(transData.expect.reference.length);
                const a = transaction.reference.sort();
                const b = transData.expect.reference.sort();
                for (let i = 0; i < a.length; i++) {
                    (0, chai_1.expect)(a[i]).to.eq(b[i]);
                }
            });
            it("Should get standardized transaction reference ", async function () {
                (0, chai_1.expect)(transaction.stdPaymentReference).to.eq(transData.expect.stdPaymentReference);
            });
            it("Should get transaction timestamp ", async function () {
                (0, chai_1.expect)(transaction.unixTimestamp).to.eq(transData.expect.unixTimestamp);
            });
            it("Should get source address ", async function () {
                (0, chai_1.expect)(transaction.sourceAddresses.length).to.eq(transData.expect.sourceAddresses.length);
                const a = transaction.sourceAddresses.sort();
                const b = transData.expect.sourceAddresses.sort();
                for (let i = 0; i < a.length; i++) {
                    (0, chai_1.expect)(a[i]).to.eq(b[i]);
                }
            });
            it("Should get receiving address ", async function () {
                (0, chai_1.expect)(transaction.receivingAddresses.length).to.eq(transData.expect.receivingAddresses.length);
                const a = transaction.receivingAddresses.sort();
                const b = transData.expect.receivingAddresses.sort();
                for (let i = 0; i < a.length; i++) {
                    (0, chai_1.expect)(a[i]).to.eq(b[i]);
                }
            });
            it("Should get fee ", async function () {
                if (transData.expect.isFeeError) {
                    (0, chai_1.expect)(function () {
                        transaction.fee;
                    }).to.throw(transData.expect.fee);
                }
                else {
                    (0, chai_1.expect)(transaction.fee.toString()).to.eq(transData.expect.fee);
                }
            });
            it("Should spend amount ", async function () {
                (0, chai_1.expect)(transaction.spentAmounts.length).to.eq(transData.expect.spentAmounts.length);
                const a = transaction.spentAmounts.sort();
                const b = transData.expect.spentAmounts.sort();
                for (let i = 0; i < a.length; i++) {
                    (0, chai_1.expect)(a[i].address).to.eq(b[i].address);
                    (0, chai_1.expect)(a[i].amount.toString()).to.eq(b[i].amount.toString());
                }
            });
            it("Should received amount ", async function () {
                (0, chai_1.expect)(transaction.receivedAmounts.length).to.eq(transData.expect.receivedAmounts.length);
                const a = transaction.receivedAmounts.sort();
                const b = transData.expect.receivedAmounts.sort();
                for (let i = 0; i < a.length; i++) {
                    (0, chai_1.expect)(a[i].address).to.eq(b[i].address);
                    (0, chai_1.expect)(a[i].amount.toString()).to.eq(b[i].amount.toString());
                }
            });
            it("Should get type ", async function () {
                (0, chai_1.expect)(transaction.type).to.eq(transData.expect.type);
            });
            it("Should check if native payment ", async function () {
                (0, chai_1.expect)(transaction.isNativePayment).to.eq(transData.expect.isNativePayment);
            });
            it("Should get currency name ", async function () {
                (0, chai_1.expect)(transaction.currencyName).to.eq(transData.expect.currencyName);
            });
            it("Should get elementary unit ", async function () {
                (0, chai_1.expect)(transaction.elementaryUnits.toString()).to.eq(transData.expect.elementaryUnits);
            });
            it("Should get success status ", async function () {
                (0, chai_1.expect)(transaction.successStatus).to.eq(transData.expect.successStatus);
            });
        });
    }
});
// TODO special transactions
// https://www.blockchain.com/btc/tx/56214420a7c4dcc4832944298d169a75e93acf9721f00656b2ee0e4d194f9970
// https://www.blockchain.com/btc/tx/055f9c6dc094cf21fa224e1eb4a54ee3cc44ae9daa8aa47f98df5c73c48997f9
//# sourceMappingURL=transaction.test.js.map